package org.feup.apm.myauthenticator

import android.annotation.SuppressLint
import android.content.ContentValues
import android.content.Context
import net.sqlcipher.Cursor
import net.sqlcipher.database.SQLiteDatabase
import net.sqlcipher.database.SQLiteOpenHelper

@SuppressLint("StaticFieldLeak")
lateinit var dbLayer: DBLayer

class DBLayer(ctx: Context) : SQLiteOpenHelper(ctx, DB_NAME, null, DB_VERSION) {
  private val passphrase = getShared(ctx, "authShared").getString("dbPass", "")

  companion object {
    private const val DB_NAME = "auth.db"
    private const val DB_VERSION = 1
  }

  override fun onCreate(db: SQLiteDatabase) {
    val createTableSQL = "CREATE TABLE IF NOT EXISTS Users (" +
        "id INTEGER PRIMARY KEY," +
        "appid TEXT, " +
        "userid TEXT, " +
        "name TEXT, " +
        "keyalias TEXT" +
      ")"
    db.execSQL(createTableSQL)
  }

  override fun onUpgrade(db: SQLiteDatabase, oldVersion: Int, newVersion: Int) {
    // Handle database upgrade if needed
  }

  fun insertUser(appid: String, userid: String, name: String, keyalias: String) {
    val values = ContentValues().apply {
      put("appid", appid)
      put("userid", userid)
      put("name", name)
      put("keyalias", keyalias)
    }
    getWritableDatabase(passphrase).insert("Users", null, values)
  }

  fun getUserByAppId(appId: String): MutableList<String> {
    val res: MutableList<String> = mutableListOf()
    val result = getReadableDatabase(passphrase).rawQuery("SELECT * FROM Users WHERE appid = ?", arrayOf(appId))
    if (result.moveToFirst()) {
      res.add(result.getString(1))
      res.add(result.getString(2))
      res.add(result.getString(3))
      res.add(result.getString(4))
    }
    return res
  }

  fun getUsers(): Cursor {
    return getReadableDatabase(passphrase).rawQuery("SELECT * FROM Users", null)
  }
}