package org.feup.apm.myauthenticator

import java.io.BufferedReader
import java.io.InputStream
import java.io.InputStreamReader
import java.io.IOException
import java.net.HttpURLConnection
import java.net.URL
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import org.json.JSONObject

private fun readStream(input: InputStream): String {
  var reader: BufferedReader? = null
  var line: String?
  val response = StringBuilder()
  try {
    reader = BufferedReader(InputStreamReader(input))
    while (reader.readLine().also{ line = it } != null)
      response.append(line)
  }
  catch (e: IOException) {
    response.clear()
    response.append("readStream: ${e.message}")
  }
  reader?.close()
  return response.toString()
}

suspend fun signupResponse(baseAddress: String, port: Int, name: String,
                           uname: String, authCert: String, userCert: String, signature: String): String {
  val urlRoute = "/auth-signup"
  val url = URL("http://$baseAddress:$port$urlRoute")
  val payload = JSONObject().apply {
    put("name", name)
    put("uid", uname)
    put("auth", authCert)
    put("user", userCert)
    put("sig", signature)
  }

  return withContext(Dispatchers.IO) {
    var urlConnection: HttpURLConnection? = null
    var result: String
    try {
      urlConnection = (url.openConnection() as HttpURLConnection).apply {
        doOutput = true
        doInput = true
        requestMethod = "POST"
        setRequestProperty("Content-Type", "application/json")
        useCaches = false
        connectTimeout = 5000
        with(outputStream) {
          write(payload.toString().toByteArray())
          flush()
          close()
        }
        // get response
        result = if (responseCode == 200)
          readStream(inputStream)
        else
          "Code: $responseCode"
      }
    } catch (e: Exception) {
      result = e.toString()
    }
    urlConnection?.disconnect()
    result
  }
}

suspend fun signinResponse(baseAddress: String, port: Int, uname: String, chaSign: String, sign: String): String {
  val urlRoute = "/auth-signin"
  val url = URL("http://$baseAddress:$port$urlRoute")
  val payload = JSONObject().apply {
    put("uid", uname)
    put("cha", chaSign)
    put("sig", sign)
  }

  return withContext(Dispatchers.IO) {
    var urlConnection: HttpURLConnection? = null
    var result: String
    try {
      urlConnection = (url.openConnection() as HttpURLConnection).apply {
        doOutput = true
        doInput = true
        requestMethod = "POST"
        setRequestProperty("Content-Type", "application/json")
        useCaches = false
        connectTimeout = 5000
        with(outputStream) {
          write(payload.toString().toByteArray())
          flush()
          close()
        }
        // get response
        result = if (responseCode == 200)
          readStream(inputStream)
        else
          "Code: $responseCode"
      }
    } catch (e: Exception) {
      result = e.toString()
    }
    urlConnection?.disconnect()
    result
  }
}